package de.mlessmann.certassist.service;

import de.mlessmann.certassist.models.Passphrase;
import de.mlessmann.certassist.repositories.PassphraseRepository;
import java.security.SecureRandom;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.lang.NonNull;
import org.springframework.security.crypto.encrypt.Encryptors;
import org.springframework.security.crypto.keygen.KeyGenerators;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor
public class PassphraseService {

    private static final String CHARACTERS =
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*()-_=+[]{}|;:,.<>?";

    public static String generateNewPassword(int length) {
        SecureRandom random = new SecureRandom();
        StringBuilder passphrase = new StringBuilder(length);
        for (int i = 0; i < length; i++) {
            passphrase.append(CHARACTERS.charAt(random.nextInt(CHARACTERS.length())));
        }
        return passphrase.toString();
    }

    @Value("${APP_KEY}")
    private final String appKey;

    private final PassphraseRepository passphraseRepository;

    @NonNull
    public Passphrase storePassphrase(@NonNull String identifier, @NonNull String passphrase) {
        String salt = KeyGenerators.string().generateKey();
        String encPassphrase = Encryptors.delux(appKey, salt).encrypt(passphrase);

        var phrase = new Passphrase();
        phrase.setIdentifier(identifier);
        phrase.setSalt(salt);
        phrase.setEncPassphrase(encPassphrase);
        return passphraseRepository.save(phrase);
    }

    @NonNull
    public Passphrase readPassphrase(String identifier) {
        var passphrase = passphraseRepository.findByIdentifierEquals(identifier);
        if (passphrase == null) {
            throw new IllegalArgumentException("No passphrase found for identifier: " + identifier);
        }

        String unencPassphrase = Encryptors.delux(appKey, passphrase.getSalt()).decrypt(passphrase.getEncPassphrase());
        passphrase.setPassphrase(unencPassphrase);
        return passphrase;
    }
}
